from pathlib import Path
import polars as pl


def clean_text(data, col: str = 'item_name') -> pl.DataFrame:
    import re

    regex = "[\(\[\<\"].*?[\)\]\>\"]"
    return data.with_columns(
        pl.col(col).map_elements(
            lambda x: re.sub(regex, "", x).lower().rstrip('.').strip()
        )
        .alias(f'{col.lower()}_clean')
    )


def ngrams_func(string):
    ngrams = zip(*[string[i:] for i in range(3)])
    return [''.join(ngram) for ngram in ngrams]


def tfidf(lst_item, dim: int = 512):
    from sklearn.feature_extraction.text import TfidfVectorizer

    vectorizer = TfidfVectorizer(analyzer=ngrams_func, max_features=dim)
    vectorizer.fit(lst_item)
    return vectorizer


def rm_all_folder(path: Path) -> None:
    for child in path.glob('*'):
        if child.is_file():
            child.unlink()
        else:
            rm_all_folder(child)
    path.rmdir()


def make_dir(folder_name: Path) -> None:
    if not folder_name.exists():
        folder_name.mkdir(parents=True, exist_ok=True)


def check_file_type(file_path: str) -> pl.DataFrame:
    if isinstance(file_path, str):
        file_path = Path(file_path).suffix[1:]
    file_type = file_path.suffix[1:]

    dict_ = {
        'csv': pl.read_csv,
        'parquet': pl.read_parquet
    }
    df = dict_[file_type](file_path)
    return df



from src.item_matching import Matching
from pathlib import Path
import polars as pl
from src.item_matching.build_index.func_img import PipelineImage


path = Path('/home/kevin/Downloads/cb')

# db
path_db = path / 'cb_2024-03-07.parquet'
pipe = PipelineImage(path)
df_db = pl.read_parquet(path_db)
df_db, df_img_db = pipe.run(df_db, mode='db', download=False)

# q
df_q = df_db.clone()
df_q.columns = [f'q_{i.split('db_')[1]}' for i in df_db.columns]

json_stats = Matching(path, df_q=df_q, df_db=df_db).run(match_mode='image', use_clean_text=False, export_type='csv')



from src.item_matching import Matching
from pathlib import Path


path = Path('/home/kevin/Downloads/yang')
path_db = path / 'fss (query)/fss_itemid_Beauty.csv'
path_q = path / 'nonfss (database)/nonfss_itemid_Beauty.csv'
json_stats = Matching(path, path_db, path_q).run(match_mode='text')



from pathlib import Path
import pandas as pd
import polars as pl
from time import perf_counter
from .build_index.func import clean_text, rm_all_folder, check_file_type, make_dir
from .build_index.matching import BELargeScale


class Matching:
    def __init__(
            self,
            col_category: str,
            path: str | Path | pl.DataFrame | pd.DataFrame,
            file_database: str | Path = None,
            file_query: str | Path = None,
            df_db: pl.DataFrame | pd.DataFrame = None,
            df_q: pl.DataFrame | pd.DataFrame = None,
    ):
        self.path = path
        self.file_database = file_database
        self.file_query = file_query
        self.df_db = df_db
        self.df_q = df_q
        self.col_category = col_category

    def clean_text(self, data: pl.DataFrame, mode: str):
        return (
            data
            .pipe(clean_text)
            .select(pl.all().name.prefix(f'{mode}_'))
            .drop_nulls()
        )

    def run(
            self,
            match_mode: str = 'text',
            export_type: str = 'parquet',
            top_k: int = 10,
    ):
        """
        Run matching processes
        :param match_mode: text | image
        :param export_type: parquet | csv
        :param top_k: top k matches
        :return: json
        """
        json_stats = {}

        # check if import df
        if self.file_database:
            self.df_db = check_file_type(self.file_database)
            self.df_q = check_file_type(self.file_query)

        # Database
        if match_mode != 'image':
            self.df_db = self.clean_text(self.df_db, 'db')
            self.df_q = self.clean_text(self.df_q, 'q')

        json_stats.update({'database shape': self.df_db.shape[0]})
        json_stats.update({'query shape': self.df_q.shape[0]})
        print(json_stats['database shape'])
        print(json_stats['query shape'])

        # Match
        be = BELargeScale(self.path, text_sparse=512)
        if match_mode == 'image':
            be = BELargeScale(self.path, img_dim=True)
        elif match_mode == 'text_dense':
            be = BELargeScale(self.path, text_dense=True)

        start = perf_counter()
        path_match_result = self.path / 'result_match'
        make_dir(path_match_result)
        for cat in sorted(self.df_q[f'q_{self.col_category}'].unique()):
            # filter cat
            file_name = path_match_result / f'{cat}.{export_type}'
            chunk_db = self.df_db.filter(pl.col(f'db_{self.col_category}') == cat)
            chunk_q = self.df_q.filter(pl.col(f'q_{self.col_category}') == cat)
            print(f'🐋 Start matching by [{match_mode}] cat: {cat} - Database shape {chunk_db.shape}, Query shape {chunk_q.shape}')

            if chunk_q.shape[0] == 0 or chunk_db.shape[0] == 0:
                print(f'Database/Query have no data')
                continue
            elif file_name.exists():
                print(f'File already exists: {file_name}')
                continue

            # match
            df_match = be.match(chunk_db, chunk_q, top_k=top_k)

            # export
            if export_type == 'parquet':
                df_match.write_parquet(file_name)
            elif export_type == 'csv':
                df_match.write_csv(file_name)

            # remove caches
            rm_all_folder(self.path / 'index')
            rm_all_folder(self.path / 'result')
            for i in ['db', 'q']:
                rm_all_folder(self.path / f'{i}_array')
                rm_all_folder(self.path / f'{i}_ds')

        # update log
        time_perf = perf_counter() - start
        json_stats.update({'time_perf': time_perf})
        json_stats.update({'path result': path_match_result})
        print(f'🐋 Your files are ready, please find here: {self.path}')
        return json_stats



from src.item_matching import Matching
from pathlib import Path
import polars as pl


path = Path('/home/kevin/Downloads/yang')
path_q = path / 'fss (query)/fss_itemid_Home & Living.csv'
path_db = path / 'nonfss (database)/nonfss_itemid_Home & Living.csv'
db = pl.read_csv(path_db)
q = pl.read_csv(path_q)
json_stats = Matching(path, df_q=q, df_db=db).run(match_mode='text')
